@@ -148,17 +148,44 @@ class MRequest(object):
         return CastResponse(self.data, target)
 
 
+class MCastRequest(object):
+    def __init__(self, sender_id, zid, message):
+        self.sender_id = sender_id
+        self.zid = zid
+        self.data = message
+        self.command = None
+        self.message = None
+        self.dispatch = None
+        logging.info('msg:%s', message)
+
+    def dispatch_response(self, target):
+        return RawResponse(self, self.data, target)
+
+
 def send_command_to(config, aws, endpoint, response):
     resp = response.make_response(config, aws)
     logging.info(aws.send_msg(endpoint, resp))
     logging.info('command %s sended', response.data)
-    if isinstance(response, MessageResponse) or isinstance(response, CastResponse):
+    if isinstance(response, MessageResponse) or isinstance(response, CastResponse) or isinstance(response, RawResponse):
         # 如果是Message消息就发APNS推送
         # msg = response.data.get('content').get('body').get('content')[:72]
         resp2 = aws.send_sns_to(response.arn, 'You have a new message')
         logging.info('push notification sended:%s with arn:%s', resp2, response.arn)
 
 
+class RawResponse(object):
+    def __init__(self, req, message, target):
+        self.request = req
+        self.message = message
+        self.target = target
+        self.endpoint = target['endpoint']
+        self.arn = target['arn']
+
+    def make_response(self, config, aws):
+        message = base64.urlsafe_b64encode(self.message)
+        return f"cast:{self.request.zid}:{message}"
+
+
 class Response(object):
     def __init__(self, basic_dict, tartget):
         self.data = basic_dict
@@ -300,6 +327,7 @@ executors = {}
 
 
 MESSAGE_HANDLER = 'MESSAGE_HANDLER'
+CAST_HANDLER = 'CAST_HANDLER'
 
 
 def api(cmd_name):
@@ -324,18 +352,20 @@ def process_message(config, message):
             logging.error(formatError(e))
     else:
         #logging.info(f"get b64:{message.encode()}")
-        raw_message = base64.decodebytes(message.encode())
-        pk = raw_message[:66]
-        #logging.info(f'key is {pk}, len:{len(pk)}')
-        bytes_data = raw_message[66:]
-        #logging.info(f'data length is:{len(bytes_data)}')
-        publicKey = secp256k1.PublicKey.restore(pk.decode())
-        try:
-            raw = privateKey.decrypt(publicKey, bytes(bytes_data), None)
-            logging.info(f"get message:${raw[:100]}")
-            req = MRequest(raw, new_version=True, enc_raw=message)
-        except Exception as e:
-            logging.error(formatError(e))
+        if message.startswith("cast:"):
+            _, zid, sender_id, _message = message.split(':')
+            return MCastRequest(zid, sender_id, _message)
+        else:
+            raw_message = base64.decodebytes(message.encode())
+            pk = raw_message[:66]
+            bytes_data = raw_message[66:]
+            publicKey = secp256k1.PublicKey.restore(pk.decode())
+            try:
+                raw = privateKey.decrypt(publicKey, bytes(bytes_data), None)
+                logging.info(f"get message:${raw[:100]}")
+                req = MRequest(raw, new_version=True, enc_raw=message)
+            except Exception as e:
+                logging.error(formatError(e))
     return req
 
 
@@ -344,7 +374,7 @@ def worker(pid, message):
     config = message.get('config')
     aws = Aws(config)
     for msg in messages:
-        body = json.loads(msg.get('Body'))
+        body = msg.get('Body')
         req = process_message(config, body)
         aws.rm_msg(config.get('ID'), msg.get('ReceiptHandle'))
         resp = None
@@ -359,6 +389,8 @@ def worker(pid, message):
             if req.dispatch:
                 if MESSAGE_HANDLER in executors:
                     resp = executors[MESSAGE_HANDLER](config, aws, req)
+            if isinstance(req, MCastRequest):
+                resp = executors[CAST_HANDLER](config, aws, req)
         except Exception as e:
             logging.error(e)
         if resp:
